@@ -4,20 +4,36 @@ use cairo_sys;
 use glib::translate::*;
 use glib_sys;
 use libc;
-use pango::{self, FontMapExt};
+use pango::{self, ContextExt, FontMapExt, LayoutExt};
 use pango_cairo_sys;
+use pango_sys;
 use pangocairo;
 use std::cell::RefCell;
+use std::rc::Rc;
 
 use bbox::BoundingBox;
 use clip_path::{ClipPathUnits, NodeClipPath};
 use coord_units::CoordUnits;
 use defs::{self, RsvgDefs};
-use filters::filter_render;
+use filters;
+use float_eq_cairo::ApproxEqCairo;
+use length::Dasharray;
 use mask::NodeMask;
-use node::{rc_node_ptr_eq, CascadedValues, NodeType, RsvgNode};
+use node::{CascadedValues, NodeType, RsvgNode};
+use paint_server::{self, PaintServer};
 use rect::RectangleExt;
-use state::{CompOp, ComputedValues, EnableBackground};
+use state::{
+    ClipRule,
+    CompOp,
+    ComputedValues,
+    EnableBackground,
+    FillRule,
+    ShapeRendering,
+    StrokeDasharray,
+    StrokeLinecap,
+    StrokeLinejoin,
+    TextRendering,
+};
 use unitinterval::UnitInterval;
 use viewbox::ViewBox;
 
@@ -193,7 +209,7 @@ impl<'a> DrawingCtx {
         self.acquired_nodes
             .borrow()
             .iter()
-            .find(|n| rc_node_ptr_eq(n, node))
+            .find(|n| Rc::ptr_eq(n, node))
             .is_some()
     }
 
@@ -238,9 +254,15 @@ impl<'a> DrawingCtx {
             original_cr.save();
 
             let clip_uri = values.clip_path.0.get();
-            let filter = values.filter.0.get();
             let mask = values.mask.0.get();
 
+            // The `filter` property does not apply to masks.
+            let filter = if node.get_type() == NodeType::Mask {
+                None
+            } else {
+                values.filter.0.get()
+            };
+
             let UnitInterval(opacity) = values.opacity.0;
             let comp_op = values.comp_op;
             let enable_background = values.enable_background;
@@ -319,27 +341,30 @@ impl<'a> DrawingCtx {
                         // actually ensure that.
                         let output = self.surfaces_stack.pop().unwrap();
 
-                        // The bbox rect can be None, for example, if a filter is applied to an
-                        // empty group. There is nothing to filter in this case.
-                        self.bbox.rect.and_then(|_| {
-                            self.get_acquired_node_of_type(filter, NodeType::Filter)
-                                .and_then(|acquired| {
-                                    let filter_node = acquired.get();
+                        match self.get_acquired_node_of_type(filter, NodeType::Filter) {
+                            Some(acquired) => {
+                                let filter_node = acquired.get();
 
-                                    if !filter_node.is_in_error() {
-                                        // FIXME: deal with out of memory here
-                                        Some(filter_render(
-                                            &filter_node,
-                                            node,
-                                            &output,
-                                            self,
-                                            "2103".as_ptr() as *const i8,
-                                        ))
-                                    } else {
-                                        None
-                                    }
-                                })
-                        })
+                                if !filter_node.is_in_error() {
+                                    // FIXME: deal with out of memory here
+                                    Some(filters::render(&filter_node, node, &output, self))
+                                } else {
+                                    None
+                                }
+                            }
+                            None => {
+                                // Non-existing filters must act as null filters (that is, an
+                                // empty surface is returned).
+                                // TODO: handle the error properly and not via expect().
+                                Some(
+                                    cairo::ImageSurface::create(
+                                        cairo::Format::ARgb32,
+                                        child_surface.get_width(),
+                                        child_surface.get_height(),
+                                    ).expect("couldn't create an empty surface"),
+                                )
+                            }
+                        }
                     })
                     .or(Some(child_surface))
                     .unwrap();
@@ -407,6 +432,185 @@ impl<'a> DrawingCtx {
         context
     }
 
+    pub fn draw_pango_layout(
+        &mut self,
+        layout: &pango::Layout,
+        values: &ComputedValues,
+        x: f64,
+        y: f64,
+        clipping: bool,
+    ) {
+        let (ink, _) = layout.get_extents();
+
+        if ink.width == 0 || ink.height == 0 {
+            return;
+        }
+
+        let cr = self.get_cairo_context();
+        cr.save();
+
+        self.set_affine_on_cr(&cr);
+
+        let affine = cr.get_matrix();
+
+        let gravity = layout.get_context().unwrap().get_gravity();
+        let bbox = compute_text_bbox(&ink, x, y, &affine, gravity);
+
+        if !clipping {
+            self.insert_bbox(&bbox);
+        }
+
+        cr.set_antialias(cairo::Antialias::from(values.text_rendering));
+
+        self.setup_cr_for_stroke(&cr, values);
+
+        let rotation = unsafe { pango_sys::pango_gravity_to_rotation(gravity.to_glib()) };
+
+        cr.move_to(x, y);
+        if !rotation.approx_eq_cairo(&0.0) {
+            cr.rotate(-rotation);
+        }
+
+        let current_color = &values.color.0;
+
+        let fill_opacity = &values.fill_opacity.0;
+
+        if !clipping {
+            if paint_server::set_source_paint_server(
+                self,
+                &values.fill.0,
+                fill_opacity,
+                &bbox,
+                current_color,
+            ) {
+                pangocairo::functions::update_layout(&cr, layout);
+                pangocairo::functions::show_layout(&cr, layout);
+            }
+        }
+
+        let stroke_opacity = &values.stroke_opacity.0;
+
+        let mut need_layout_path = clipping;
+
+        if !clipping {
+            if paint_server::set_source_paint_server(
+                self,
+                &values.stroke.0,
+                stroke_opacity,
+                &bbox,
+                &current_color,
+            ) {
+                need_layout_path = true;
+            }
+        }
+
+        if need_layout_path {
+            pangocairo::functions::update_layout(&cr, layout);
+            pangocairo::functions::layout_path(&cr, layout);
+
+            if !clipping {
+                let ib = BoundingBox::new(&affine).with_ink_extents(cr.stroke_extents());
+                cr.stroke();
+                self.insert_bbox(&ib);
+            }
+        }
+
+        cr.restore();
+    }
+
+    fn setup_cr_for_stroke(&self, cr: &cairo::Context, values: &ComputedValues) {
+        cr.set_line_width(values.stroke_width.0.normalize(values, self));
+        cr.set_miter_limit(values.stroke_miterlimit.0);
+        cr.set_line_cap(cairo::LineCap::from(values.stroke_line_cap));
+        cr.set_line_join(cairo::LineJoin::from(values.stroke_line_join));
+
+        if let StrokeDasharray(Dasharray::Array(ref dashes)) = values.stroke_dasharray {
+            let normalized_dashes: Vec<f64> =
+                dashes.iter().map(|l| l.normalize(values, self)).collect();
+
+            let total_length = normalized_dashes.iter().fold(0.0, |acc, &len| acc + len);
+
+            if total_length > 0.0 {
+                let offset = values.stroke_dashoffset.0.normalize(values, self);
+                cr.set_dash(&normalized_dashes, offset);
+            } else {
+                cr.set_dash(&[], 0.0);
+            }
+        }
+    }
+
+    pub fn stroke_and_fill(&mut self, cr: &cairo::Context, values: &ComputedValues) {
+        cr.set_antialias(cairo::Antialias::from(values.shape_rendering));
+
+        self.setup_cr_for_stroke(cr, values);
+
+        // Update the bbox in the rendering context.  Below, we actually set the
+        // fill/stroke patterns on the cairo_t.  That process requires the
+        // rendering context to have an updated bbox; for example, for the
+        // coordinate system in patterns.
+        let bbox = compute_stroke_and_fill_box(cr, values);
+        self.insert_bbox(&bbox);
+
+        let current_color = &values.color.0;
+
+        let fill_opacity = &values.fill_opacity.0;
+
+        if paint_server::set_source_paint_server(
+            self,
+            &values.fill.0,
+            fill_opacity,
+            &bbox,
+            current_color,
+        ) {
+            if values.stroke.0 == PaintServer::None {
+                cr.fill();
+            } else {
+                cr.fill_preserve();
+            }
+        }
+
+        let stroke_opacity = values.stroke_opacity.0;
+
+        if paint_server::set_source_paint_server(
+            self,
+            &values.stroke.0,
+            &stroke_opacity,
+            &bbox,
+            &current_color,
+        ) {
+            cr.stroke();
+        }
+
+        // clear the path in case stroke == fill == None; otherwise
+        // we leave it around from computing the bounding box
+        cr.new_path();
+    }
+
+    pub fn set_affine_on_cr(&self, cr: &cairo::Context) {
+        let (x0, y0) = self.get_offset();
+        let affine = cr.get_matrix();
+        let matrix = cairo::Matrix::new(
+            affine.xx,
+            affine.yx,
+            affine.xy,
+            affine.yy,
+            affine.x0 + x0,
+            affine.y0 + y0,
+        );
+        cr.set_matrix(matrix);
+    }
+
+    pub fn clip(&mut self, x: f64, y: f64, w: f64, h: f64) {
+        let cr = self.get_cairo_context();
+        let save_affine = cr.get_matrix();
+
+        self.set_affine_on_cr(&cr);
+
+        cr.rectangle(x, y, w, h);
+        cr.clip();
+        cr.set_matrix(save_affine);
+    }
+
     pub fn draw_node_on_surface(
         &mut self,
         node: &RsvgNode,
@@ -448,7 +652,7 @@ impl<'a> DrawingCtx {
         let stack_top = self.drawsub_stack.pop();
 
         if let Some(ref top) = stack_top {
-            if !rc_node_ptr_eq(&top, node) {
+            if !Rc::ptr_eq(top, node) {
                 draw = false;
             }
         }
@@ -490,6 +694,192 @@ fn set_font_options(context: &pango::Context, options: &cairo::FontOptions) {
     }
 }
 
+// FIXME: should the pango crate provide this like PANGO_GRAVITY_IS_VERTICAL() ?
+fn gravity_is_vertical(gravity: pango::Gravity) -> bool {
+    match gravity {
+        pango::Gravity::East | pango::Gravity::West => true,
+        _ => false,
+    }
+}
+
+fn compute_text_bbox(
+    ink: &pango::Rectangle,
+    x: f64,
+    y: f64,
+    affine: &cairo::Matrix,
+    gravity: pango::Gravity,
+) -> BoundingBox {
+    let pango_scale = f64::from(pango::SCALE);
+
+    let mut bbox = BoundingBox::new(affine);
+
+    let ink_x = f64::from(ink.x);
+    let ink_y = f64::from(ink.y);
+    let ink_width = f64::from(ink.width);
+    let ink_height = f64::from(ink.height);
+
+    if gravity_is_vertical(gravity) {
+        bbox.rect = Some(cairo::Rectangle {
+            x: x + (ink_x - ink_height) / pango_scale,
+            y: y + ink_y / pango_scale,
+            width: ink_height / pango_scale,
+            height: ink_width / pango_scale,
+        });
+    } else {
+        bbox.rect = Some(cairo::Rectangle {
+            x: x + ink_x / pango_scale,
+            y: y + ink_y / pango_scale,
+            width: ink_width / pango_scale,
+            height: ink_height / pango_scale,
+        });
+    }
+
+    bbox
+}
+
+fn compute_stroke_and_fill_box(cr: &cairo::Context, values: &ComputedValues) -> BoundingBox {
+    let affine = cr.get_matrix();
+
+    let mut bbox = BoundingBox::new(&affine);
+
+    // Dropping the precision of cairo's bezier subdivision, yielding 2x
+    // _rendering_ time speedups, are these rather expensive operations
+    // really needed here? */
+    let backup_tolerance = cr.get_tolerance();
+    cr.set_tolerance(1.0);
+
+    // Bounding box for fill
+    //
+    // Unlike the case for stroke, for fills we always compute the bounding box.
+    // In GNOME we have SVGs for symbolic icons where each icon has a bounding
+    // rectangle with no fill and no stroke, and inside it there are the actual
+    // paths for the icon's shape.  We need to be able to compute the bounding
+    // rectangle's extents, even when it has no fill nor stroke.
+
+    let fb = BoundingBox::new(&affine).with_ink_extents(cr.fill_extents());
+    bbox.insert(&fb);
+
+    // Bounding box for stroke
+
+    if values.stroke.0 != PaintServer::None {
+        let sb = BoundingBox::new(&affine).with_ink_extents(cr.stroke_extents());
+        bbox.insert(&sb);
+    }
+
+    // objectBoundingBox
+
+    let ob = BoundingBox::new(&affine).with_extents(path_extents(cr));
+    bbox.insert(&ob);
+
+    // restore tolerance
+
+    cr.set_tolerance(backup_tolerance);
+
+    bbox
+}
+
+// remove this binding once cairo-rs has Context::path_extents()
+fn path_extents(cr: &cairo::Context) -> (f64, f64, f64, f64) {
+    let mut x1: f64 = 0.0;
+    let mut y1: f64 = 0.0;
+    let mut x2: f64 = 0.0;
+    let mut y2: f64 = 0.0;
+
+    unsafe {
+        cairo_sys::cairo_path_extents(cr.to_glib_none().0, &mut x1, &mut y1, &mut x2, &mut y2);
+    }
+    (x1, y1, x2, y2)
+}
+
+impl From<StrokeLinejoin> for cairo::LineJoin {
+    fn from(j: StrokeLinejoin) -> cairo::LineJoin {
+        match j {
+            StrokeLinejoin::Miter => cairo::LineJoin::Miter,
+            StrokeLinejoin::Round => cairo::LineJoin::Round,
+            StrokeLinejoin::Bevel => cairo::LineJoin::Bevel,
+        }
+    }
+}
+
+impl From<StrokeLinecap> for cairo::LineCap {
+    fn from(j: StrokeLinecap) -> cairo::LineCap {
+        match j {
+            StrokeLinecap::Butt => cairo::LineCap::Butt,
+            StrokeLinecap::Round => cairo::LineCap::Round,
+            StrokeLinecap::Square => cairo::LineCap::Square,
+        }
+    }
+}
+
+impl From<CompOp> for cairo::Operator {
+    fn from(op: CompOp) -> cairo::Operator {
+        match op {
+            CompOp::Clear => cairo::Operator::Clear,
+            CompOp::Src => cairo::Operator::Source,
+            CompOp::Dst => cairo::Operator::Dest,
+            CompOp::SrcOver => cairo::Operator::Over,
+            CompOp::DstOver => cairo::Operator::DestOver,
+            CompOp::SrcIn => cairo::Operator::In,
+            CompOp::DstIn => cairo::Operator::DestIn,
+            CompOp::SrcOut => cairo::Operator::Out,
+            CompOp::DstOut => cairo::Operator::DestOut,
+            CompOp::SrcAtop => cairo::Operator::Atop,
+            CompOp::DstAtop => cairo::Operator::DestAtop,
+            CompOp::Xor => cairo::Operator::Xor,
+            CompOp::Plus => cairo::Operator::Add,
+            CompOp::Multiply => cairo::Operator::Multiply,
+            CompOp::Screen => cairo::Operator::Screen,
+            CompOp::Overlay => cairo::Operator::Overlay,
+            CompOp::Darken => cairo::Operator::Darken,
+            CompOp::Lighten => cairo::Operator::Lighten,
+            CompOp::ColorDodge => cairo::Operator::ColorDodge,
+            CompOp::ColorBurn => cairo::Operator::ColorBurn,
+            CompOp::HardLight => cairo::Operator::HardLight,
+            CompOp::SoftLight => cairo::Operator::SoftLight,
+            CompOp::Difference => cairo::Operator::Difference,
+            CompOp::Exclusion => cairo::Operator::Exclusion,
+        }
+    }
+}
+
+impl From<ClipRule> for cairo::FillRule {
+    fn from(c: ClipRule) -> cairo::FillRule {
+        match c {
+            ClipRule::NonZero => cairo::FillRule::Winding,
+            ClipRule::EvenOdd => cairo::FillRule::EvenOdd,
+        }
+    }
+}
+
+impl From<FillRule> for cairo::FillRule {
+    fn from(f: FillRule) -> cairo::FillRule {
+        match f {
+            FillRule::NonZero => cairo::FillRule::Winding,
+            FillRule::EvenOdd => cairo::FillRule::EvenOdd,
+        }
+    }
+}
+
+impl From<ShapeRendering> for cairo::Antialias {
+    fn from(sr: ShapeRendering) -> cairo::Antialias {
+        match sr {
+            ShapeRendering::Auto | ShapeRendering::GeometricPrecision => cairo::Antialias::Default,
+            ShapeRendering::OptimizeSpeed | ShapeRendering::CrispEdges => cairo::Antialias::None,
+        }
+    }
+}
+
+impl From<TextRendering> for cairo::Antialias {
+    fn from(tr: TextRendering) -> cairo::Antialias {
+        match tr {
+            TextRendering::Auto
+            | TextRendering::OptimizeLegibility
+            | TextRendering::GeometricPrecision => cairo::Antialias::Default,
+            TextRendering::OptimizeSpeed => cairo::Antialias::None,
+        }
+    }
+}
+
 #[no_mangle]
 pub extern "C" fn rsvg_drawing_ctx_draw_node_from_stack(
     raw_draw_ctx: *mut RsvgDrawingCtx,
@@ -562,7 +952,7 @@ impl Drop for AcquiredNode {
     fn drop(&mut self) {
         unsafe {
             let mut v = (*self.0).borrow_mut();
-            assert!(rc_node_ptr_eq(v.last().unwrap(), &self.1));
+            assert!(Rc::ptr_eq(v.last().unwrap(), &self.1));
             v.pop();
         }
     }
