@@ -6,14 +6,14 @@ use attributes::Attribute;
 use drawing_ctx::DrawingCtx;
 use error::NodeError;
 use handle::RsvgHandle;
-use node::{NodeResult, NodeTrait, RsvgCNodeImpl, RsvgNode};
+use node::{NodeResult, NodeTrait, RsvgNode};
 use parsers::ParseError;
 use property_bag::PropertyBag;
 use surface_utils::shared_surface::SharedImageSurface;
 
 use super::context::{FilterContext, FilterOutput, FilterResult};
 use super::input::Input;
-use super::{make_result, Filter, FilterError, PrimitiveWithInput};
+use super::{Filter, FilterError, PrimitiveWithInput};
 
 /// Enumeration of the possible blending modes.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
@@ -65,11 +65,6 @@ impl NodeTrait for Blend {
 
         Ok(())
     }
-
-    #[inline]
-    fn get_c_impl(&self) -> *const RsvgCNodeImpl {
-        self.base.get_c_impl()
-    }
 }
 
 impl Filter for Blend {
@@ -79,8 +74,8 @@ impl Filter for Blend {
         ctx: &FilterContext,
         draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterResult, FilterError> {
-        let input = make_result(self.base.get_input(ctx, draw_ctx))?;
-        let input_2 = make_result(ctx.get_input(draw_ctx, self.in2.borrow().as_ref()))?;
+        let input = self.base.get_input(ctx, draw_ctx)?;
+        let input_2 = ctx.get_input(draw_ctx, self.in2.borrow().as_ref())?;
         let bounds = self
             .base
             .get_bounds(ctx)
@@ -91,7 +86,7 @@ impl Filter for Blend {
         let output_surface = input_2
             .surface()
             .copy_surface(bounds)
-            .map_err(FilterError::OutputSurfaceCreation)?;
+            .map_err(FilterError::IntermediateSurfaceCreation)?;
         {
             let cr = cairo::Context::new(&output_surface);
             cr.rectangle(
@@ -110,14 +105,15 @@ impl Filter for Blend {
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: SharedImageSurface::new(output_surface).unwrap(),
+                surface: SharedImageSurface::new(output_surface)
+                    .map_err(FilterError::BadIntermediateSurfaceStatus)?,
                 bounds,
             },
         })
     }
 
     #[inline]
-    fn is_affected_by_color_interpolation_filters() -> bool {
+    fn is_affected_by_color_interpolation_filters(&self) -> bool {
         true
     }
 }
