@@ -7,7 +7,7 @@ use attributes::Attribute;
 use drawing_ctx::DrawingCtx;
 use error::NodeError;
 use handle::RsvgHandle;
-use node::{NodeResult, NodeTrait, NodeType, RsvgCNodeImpl, RsvgNode};
+use node::{NodeResult, NodeTrait, NodeType, RsvgNode};
 use parsers::{self, ListLength, NumberListError, ParseError};
 use property_bag::PropertyBag;
 use surface_utils::{
@@ -19,7 +19,7 @@ use surface_utils::{
 use util::clamp;
 
 use super::context::{FilterContext, FilterOutput, FilterResult};
-use super::{make_result, Filter, FilterError, PrimitiveWithInput};
+use super::{Filter, FilterError, PrimitiveWithInput};
 
 /// The `feComponentTransfer` filter primitive.
 pub struct ComponentTransfer {
@@ -211,11 +211,6 @@ impl NodeTrait for ComponentTransfer {
     ) -> NodeResult {
         self.base.set_atts(node, handle, pbag)
     }
-
-    #[inline]
-    fn get_c_impl(&self) -> *const RsvgCNodeImpl {
-        self.base.get_c_impl()
-    }
 }
 
 impl NodeTrait for FuncX {
@@ -283,7 +278,7 @@ impl Filter for ComponentTransfer {
         ctx: &FilterContext,
         draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterResult, FilterError> {
-        let input = make_result(self.base.get_input(ctx, draw_ctx))?;
+        let input = self.base.get_input(ctx, draw_ctx)?;
         let bounds = self
             .base
             .get_bounds(ctx)
@@ -295,7 +290,7 @@ impl Filter for ComponentTransfer {
             cairo::Format::ARgb32,
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
-        ).map_err(FilterError::OutputSurfaceCreation)?;
+        ).map_err(FilterError::IntermediateSurfaceCreation)?;
 
         // Enumerate all child <feFuncX> nodes.
         let functions = node
@@ -377,13 +372,14 @@ impl Filter for ComponentTransfer {
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: SharedImageSurface::new(output_surface).unwrap(),
+                surface: SharedImageSurface::new(output_surface)
+                    .map_err(FilterError::BadIntermediateSurfaceStatus)?,
                 bounds,
             },
         })
     }
 
-    fn is_affected_by_color_interpolation_filters() -> bool {
+    fn is_affected_by_color_interpolation_filters(&self) -> bool {
         true
     }
 }
