@@ -7,7 +7,7 @@ use attributes::Attribute;
 use drawing_ctx::DrawingCtx;
 use error::{AttributeError, NodeError};
 use handle::RsvgHandle;
-use node::{NodeResult, NodeTrait, RsvgCNodeImpl, RsvgNode};
+use node::{NodeResult, NodeTrait, RsvgNode};
 use parsers::{self, parse, Parse};
 use property_bag::PropertyBag;
 use surface_utils::{
@@ -20,7 +20,7 @@ use util::clamp;
 
 use super::context::{FilterContext, FilterOutput, FilterResult};
 use super::input::Input;
-use super::{make_result, Filter, FilterError, PrimitiveWithInput};
+use super::{Filter, FilterError, PrimitiveWithInput};
 
 /// Enumeration of the possible compositing operations.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
@@ -93,11 +93,6 @@ impl NodeTrait for Composite {
 
         Ok(())
     }
-
-    #[inline]
-    fn get_c_impl(&self) -> *const RsvgCNodeImpl {
-        self.base.get_c_impl()
-    }
 }
 
 impl Filter for Composite {
@@ -107,8 +102,8 @@ impl Filter for Composite {
         ctx: &FilterContext,
         draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterResult, FilterError> {
-        let input = make_result(self.base.get_input(ctx, draw_ctx))?;
-        let input_2 = make_result(ctx.get_input(draw_ctx, self.in2.borrow().as_ref()))?;
+        let input = self.base.get_input(ctx, draw_ctx)?;
+        let input_2 = ctx.get_input(draw_ctx, self.in2.borrow().as_ref())?;
         let bounds = self
             .base
             .get_bounds(ctx)
@@ -121,7 +116,7 @@ impl Filter for Composite {
                 cairo::Format::ARgb32,
                 input.surface().width(),
                 input.surface().height(),
-            ).map_err(FilterError::OutputSurfaceCreation)?;
+            ).map_err(FilterError::IntermediateSurfaceCreation)?;
 
             let output_stride = output_surface.get_stride() as usize;
             {
@@ -170,7 +165,7 @@ impl Filter for Composite {
             let output_surface = input_2
                 .surface()
                 .copy_surface(bounds)
-                .map_err(FilterError::OutputSurfaceCreation)?;
+                .map_err(FilterError::IntermediateSurfaceCreation)?;
 
             let cr = cairo::Context::new(&output_surface);
             cr.rectangle(
@@ -191,14 +186,15 @@ impl Filter for Composite {
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: SharedImageSurface::new(output_surface).unwrap(),
+                surface: SharedImageSurface::new(output_surface)
+                    .map_err(FilterError::BadIntermediateSurfaceStatus)?,
                 bounds,
             },
         })
     }
 
     #[inline]
-    fn is_affected_by_color_interpolation_filters() -> bool {
+    fn is_affected_by_color_interpolation_filters(&self) -> bool {
         true
     }
 }
