@@ -1,32 +1,23 @@
 use std::cell::UnsafeCell;
 use std::collections::HashMap;
-use std::{mem, ptr};
+use std::f64;
 
-use cairo::prelude::SurfaceExt;
 use cairo::{self, MatrixTrait};
-use cairo_sys::cairo_surface_t;
-use glib::translate::{from_glib_full, from_glib_none};
-use glib_sys::*;
 
 use bbox::BoundingBox;
 use coord_units::CoordUnits;
-use drawing_ctx::{DrawingCtx, RsvgDrawingCtx};
+use drawing_ctx::DrawingCtx;
 use length::RsvgLength;
 use node::RsvgNode;
 use paint_server::{self, PaintServer};
 use srgb::{linearize_surface, unlinearize_surface};
-use surface_utils::{iterators::Pixels, shared_surface::SharedImageSurface, Pixel};
+use surface_utils::shared_surface::SharedImageSurface;
 use unitinterval::UnitInterval;
 
-use super::bounds::BoundsBuilder;
+use super::error::FilterError;
 use super::input::Input;
 use super::node::NodeFilter;
-use super::RsvgFilterPrimitive;
 
-// Required by the C code until all filters are ported to Rust.
-// Keep this in sync with
-// ../../librsvg/librsvg/rsvg-filter.h:RsvgIRect
-#[repr(C)]
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct IRect {
     pub x0: i32,
@@ -35,15 +26,6 @@ pub struct IRect {
     pub y1: i32,
 }
 
-// Required by the C code until all filters are ported to Rust.
-// Keep this in sync with
-// ../../librsvg/librsvg/filters/common.h:_RsvgFilterPrimitiveOutput
-#[repr(C)]
-pub struct RsvgFilterPrimitiveOutput {
-    surface: *mut cairo_surface_t,
-    bounds: IRect,
-}
-
 /// A filter primitive output.
 #[derive(Debug, Clone)]
 pub struct FilterOutput {
@@ -73,8 +55,6 @@ pub enum FilterInput {
     PrimitiveOutput(FilterOutput),
 }
 
-pub type RsvgFilterContext = FilterContext;
-
 /// The filter rendering context.
 pub struct FilterContext {
     /// The <filter> node.
@@ -88,7 +68,7 @@ pub struct FilterContext {
     /// Surfaces of the previous filter primitives by name.
     previous_results: HashMap<String, FilterOutput>,
     /// The background surface. Computed lazily.
-    background_surface: UnsafeCell<Option<Result<SharedImageSurface, cairo::Status>>>,
+    background_surface: UnsafeCell<Option<Result<SharedImageSurface, FilterError>>>,
     /// The filter effects region.
     effects_region: BoundingBox,
     /// Whether the currently rendered filter primitive uses linear RGB for color operations.
@@ -112,35 +92,12 @@ pub struct FilterContext {
     /// This is to be used in conjunction with setting the viewbox size to account for the scaling.
     /// For `filterUnits == userSpaceOnUse`, the viewbox will have the actual resolution size, and
     /// for `filterUnits == objectBoundingBox`, the viewbox will have the size of 1, 1.
-    affine: cairo::Matrix,
+    _affine: cairo::Matrix,
 
     /// The filter primitive affine matrix.
     ///
-    /// See the comments for `affine`, they largely apply here.
+    /// See the comments for `_affine`, they largely apply here.
     paffine: cairo::Matrix,
-
-    /// Obsolete; remove when all filters are ported to Rust.
-    channelmap: [i32; 4],
-}
-
-/// Returns a surface with black background and alpha channel matching the input surface.
-fn extract_alpha(
-    surface: &SharedImageSurface,
-    bounds: IRect,
-) -> Result<cairo::ImageSurface, cairo::Status> {
-    let mut output_surface =
-        cairo::ImageSurface::create(cairo::Format::ARgb32, surface.width(), surface.height())?;
-
-    let output_stride = output_surface.get_stride() as usize;
-    {
-        let mut output_data = output_surface.get_data().unwrap();
-
-        for (x, y, Pixel { a, .. }) in Pixels::new(surface, bounds) {
-            output_data[y as usize * output_stride + x as usize * 4 + 3] = a;
-        }
-    }
-
-    Ok(output_surface)
 }
 
 /// Computes and returns the filter effects region.
@@ -220,6 +177,19 @@ impl IRect {
     pub fn contains(self, x: i32, y: i32) -> bool {
         x >= self.x0 && x < self.x1 && y >= self.y0 && y < self.y1
     }
+
+    /// Returns an `IRect` scaled by the given amounts.
+    ///
+    /// The returned `IRect` encompasses all, even partially covered, pixels after the scaling.
+    #[inline]
+    pub fn scale(self, x: f64, y: f64) -> IRect {
+        IRect {
+            x0: (f64::from(self.x0) * x).floor() as i32,
+            y0: (f64::from(self.y0) * y).floor() as i32,
+            x1: (f64::from(self.x1) * x).ceil() as i32,
+            y1: (f64::from(self.y1) * y).ceil() as i32,
+        }
+    }
 }
 
 impl FilterContext {
@@ -229,11 +199,18 @@ impl FilterContext {
         node_being_filtered: &RsvgNode,
         source_surface: SharedImageSurface,
         draw_ctx: &mut DrawingCtx,
-        channelmap: [i32; 4],
     ) -> Self {
         let cr_affine = draw_ctx.get_cairo_context().get_matrix();
         let bbox = draw_ctx.get_bbox().clone();
-        let bbox_rect = bbox.rect.unwrap();
+
+        // The rect can be empty (for example, if the filter is applied to an empty group).
+        // However, with userSpaceOnUse it's still possible to create images with a filter.
+        let bbox_rect = bbox.rect.unwrap_or(cairo::Rectangle {
+            x: 0.0,
+            y: 0.0,
+            width: 0.0,
+            height: 0.0,
+        });
 
         let filter = filter_node.get_impl::<NodeFilter>().unwrap();
 
@@ -270,7 +247,7 @@ impl FilterContext {
         let width = source_surface.width();
         let height = source_surface.height();
 
-        let mut rv = Self {
+        Self {
             node: filter_node.clone(),
             node_being_filtered: node_being_filtered.clone(),
             source_surface,
@@ -286,18 +263,9 @@ impl FilterContext {
                 f64::from(height),
             ),
             processing_linear_rgb: false,
-            affine,
+            _affine: affine,
             paffine,
-            channelmap,
-        };
-
-        let last_result = FilterOutput {
-            surface: rv.source_surface.clone(),
-            bounds: rv.effects_region().rect.unwrap().into(),
-        };
-
-        rv.last_result = Some(last_result);
-        rv
+        }
     }
 
     /// Returns the <filter> node for this context.
@@ -326,8 +294,10 @@ impl FilterContext {
 
     /// Returns the surface containing the source graphic alpha.
     #[inline]
-    pub fn source_alpha(&self, bounds: IRect) -> Result<cairo::ImageSurface, cairo::Status> {
-        extract_alpha(self.source_graphic(), bounds)
+    pub fn source_alpha(&self, bounds: IRect) -> Result<cairo::ImageSurface, FilterError> {
+        self.source_surface
+            .extract_alpha(bounds)
+            .map_err(FilterError::IntermediateSurfaceCreation)
     }
 
     /// Computes and returns the background image snapshot.
@@ -344,8 +314,11 @@ impl FilterContext {
         let (x, y) = draw_ctx.get_raw_offset();
         let stack = draw_ctx.get_cr_stack();
 
+        // TODO: as far as I can tell this should not render elements past the last (topmost) one
+        // with enable-background: new (because technically we shouldn't have been caching them).
+        // Right now there are no enable-background checks whatsoever.
         let cr = cairo::Context::new(&surface);
-        for draw in stack.into_iter().rev() {
+        for draw in stack.into_iter() {
             let nested = draw_ctx.is_cairo_context_nested(&draw);
             cr.set_source_surface(
                 &draw.get_target(),
@@ -362,7 +335,7 @@ impl FilterContext {
     pub fn background_image(
         &self,
         draw_ctx: &DrawingCtx,
-    ) -> Result<&SharedImageSurface, cairo::Status> {
+    ) -> Result<&SharedImageSurface, FilterError> {
         {
             // At this point either no, or only immutable references to background_surface exist, so
             // it's ok to make an immutable reference.
@@ -381,7 +354,11 @@ impl FilterContext {
 
         *bg = Some(
             self.compute_background_image(draw_ctx)
-                .and_then(SharedImageSurface::new),
+                .map_err(FilterError::IntermediateSurfaceCreation)
+                .and_then(|surface| {
+                    SharedImageSurface::new(surface)
+                        .map_err(FilterError::BadIntermediateSurfaceStatus)
+                }),
         );
 
         // Return the only existing reference as immutable.
@@ -394,9 +371,10 @@ impl FilterContext {
         &self,
         draw_ctx: &DrawingCtx,
         bounds: IRect,
-    ) -> Result<cairo::ImageSurface, cairo::Status> {
-        self.background_image(draw_ctx)
-            .and_then(|surface| extract_alpha(surface, bounds))
+    ) -> Result<cairo::ImageSurface, FilterError> {
+        self.background_image(draw_ctx)?
+            .extract_alpha(bounds)
+            .map_err(FilterError::IntermediateSurfaceCreation)
     }
 
     /// Returns the output of the filter primitive by its result name.
@@ -408,21 +386,34 @@ impl FilterContext {
     /// Converts this `FilterContext` into the surface corresponding to the output of the filter
     /// chain.
     #[inline]
-    pub fn into_output(self) -> SharedImageSurface {
-        self.last_result
-            .map(|FilterOutput { surface, .. }| surface)
-            .unwrap_or(self.source_surface)
+    pub fn into_output(self) -> Result<SharedImageSurface, FilterError> {
+        match self.last_result.map(|FilterOutput { surface, .. }| surface) {
+            Some(surface) => Ok(surface),
+            None => {
+                let empty_surface = cairo::ImageSurface::create(
+                    cairo::Format::ARgb32,
+                    self.source_surface.width(),
+                    self.source_surface.height(),
+                ).map_err(FilterError::IntermediateSurfaceCreation)?;
+
+                SharedImageSurface::new(empty_surface)
+                    .map_err(FilterError::BadIntermediateSurfaceStatus)
+            }
+        }
     }
 
     /// Stores a filter primitive result into the context.
     #[inline]
-    pub fn store_result(&mut self, mut result: FilterResult) {
+    pub fn store_result(&mut self, mut result: FilterResult) -> Result<(), FilterError> {
         // Unlinearize the surface if needed.
         if self.processing_linear_rgb {
-            // TODO: unwrap() on unlinearize_surface() can panic if we run out of memory for Cairo.
-            result.output.surface = SharedImageSurface::new(
-                unlinearize_surface(&result.output.surface, result.output.bounds).unwrap(),
-            ).unwrap();
+            result.output.surface =
+                unlinearize_surface(&result.output.surface, result.output.bounds)
+                    .map_err(FilterError::IntermediateSurfaceCreation)
+                    .and_then(|surface| {
+                        SharedImageSurface::new(surface)
+                            .map_err(FilterError::BadIntermediateSurfaceStatus)
+                    })?;
         }
 
         if let Some(name) = result.name {
@@ -430,6 +421,7 @@ impl FilterContext {
         }
 
         self.last_result = Some(result.output);
+        Ok(())
     }
 
     /// Returns the paffine matrix.
@@ -509,16 +501,19 @@ impl FilterContext {
     /// Retrieves the filter input surface according to the SVG rules.
     ///
     /// Does not take `processing_linear_rgb` into account.
-    // TODO: pass the errors through.
-    fn get_input_raw(&self, draw_ctx: &mut DrawingCtx, in_: Option<&Input>) -> Option<FilterInput> {
+    fn get_input_raw(
+        &self,
+        draw_ctx: &mut DrawingCtx,
+        in_: Option<&Input>,
+    ) -> Result<FilterInput, FilterError> {
         if in_.is_none() {
             // No value => use the last result.
             // As per the SVG spec, if the filter primitive is the first in the chain, return the
             // source graphic.
             if let Some(output) = self.last_result().cloned() {
-                return Some(FilterInput::PrimitiveOutput(output));
+                return Ok(FilterInput::PrimitiveOutput(output));
             } else {
-                return Some(FilterInput::StandardInput(self.source_graphic().clone()));
+                return Ok(FilterInput::StandardInput(self.source_graphic().clone()));
             }
         }
 
@@ -526,48 +521,62 @@ impl FilterContext {
         let values = cascaded.get();
 
         match *in_.unwrap() {
-            Input::SourceGraphic => Some(FilterInput::StandardInput(self.source_graphic().clone())),
+            Input::SourceGraphic => Ok(FilterInput::StandardInput(self.source_graphic().clone())),
             Input::SourceAlpha => self
                 .source_alpha(self.effects_region().rect.unwrap().into())
-                .ok()
-                .map(|surface| SharedImageSurface::new(surface).unwrap())
+                .and_then(|surface| {
+                    SharedImageSurface::new(surface)
+                        .map_err(FilterError::BadIntermediateSurfaceStatus)
+                })
                 .map(FilterInput::StandardInput),
             Input::BackgroundImage => self
                 .background_image(draw_ctx)
-                .ok()
-                .cloned()
+                .map(Clone::clone)
                 .map(FilterInput::StandardInput),
             Input::BackgroundAlpha => self
                 .background_alpha(draw_ctx, self.effects_region().rect.unwrap().into())
-                .ok()
-                .map(|surface| SharedImageSurface::new(surface).unwrap())
+                .and_then(|surface| {
+                    SharedImageSurface::new(surface)
+                        .map_err(FilterError::BadIntermediateSurfaceStatus)
+                })
                 .map(FilterInput::StandardInput),
 
             Input::FillPaint => self
                 .get_paint_server_surface(draw_ctx, &values.fill.0, values.fill_opacity.0)
-                .ok()
-                .map(|surface| SharedImageSurface::new(surface).unwrap())
+                .map_err(FilterError::IntermediateSurfaceCreation)
+                .and_then(|surface| {
+                    SharedImageSurface::new(surface)
+                        .map_err(FilterError::BadIntermediateSurfaceStatus)
+                })
                 .map(FilterInput::StandardInput),
             Input::StrokePaint => self
                 .get_paint_server_surface(draw_ctx, &values.stroke.0, values.stroke_opacity.0)
-                .ok()
-                .map(|surface| SharedImageSurface::new(surface).unwrap())
+                .map_err(FilterError::IntermediateSurfaceCreation)
+                .and_then(|surface| {
+                    SharedImageSurface::new(surface)
+                        .map_err(FilterError::BadIntermediateSurfaceStatus)
+                })
                 .map(FilterInput::StandardInput),
 
             Input::FilterOutput(ref name) => self
                 .filter_output(name)
                 .cloned()
-                .map(FilterInput::PrimitiveOutput),
+                .map(FilterInput::PrimitiveOutput)
+                .ok_or(FilterError::InvalidInput),
         }
     }
 
     /// Retrieves the filter input surface according to the SVG rules.
-    pub fn get_input(&self, draw_ctx: &mut DrawingCtx, in_: Option<&Input>) -> Option<FilterInput> {
-        let raw = self.get_input_raw(draw_ctx, in_);
+    pub fn get_input(
+        &self,
+        draw_ctx: &mut DrawingCtx,
+        in_: Option<&Input>,
+    ) -> Result<FilterInput, FilterError> {
+        let raw = self.get_input_raw(draw_ctx, in_)?;
 
         // Linearize the returned surface if needed.
-        if raw.is_some() && self.processing_linear_rgb {
-            let (surface, bounds) = match raw.as_ref().unwrap() {
+        if self.processing_linear_rgb {
+            let (surface, bounds) = match raw {
                 FilterInput::StandardInput(ref surface) => {
                     (surface, self.effects_region().rect.unwrap().into())
                 }
@@ -578,16 +587,19 @@ impl FilterContext {
             };
 
             linearize_surface(surface, bounds)
-                .ok()
-                .map(|surface| SharedImageSurface::new(surface).unwrap())
-                .map(|surface| match raw.as_ref().unwrap() {
+                .map_err(FilterError::IntermediateSurfaceCreation)
+                .and_then(|surface| {
+                    SharedImageSurface::new(surface)
+                        .map_err(FilterError::BadIntermediateSurfaceStatus)
+                })
+                .map(|surface| match raw {
                     FilterInput::StandardInput(_) => FilterInput::StandardInput(surface),
-                    FilterInput::PrimitiveOutput(output) => {
+                    FilterInput::PrimitiveOutput(ref output) => {
                         FilterInput::PrimitiveOutput(FilterOutput { surface, ..*output })
                     }
                 })
         } else {
-            raw
+            Ok(raw)
         }
     }
 
@@ -599,6 +611,12 @@ impl FilterContext {
         self.processing_linear_rgb = false;
         rv
     }
+
+    /// Applies the `primitiveUnits` coordinate transformation to a non-x or y distance.
+    #[inline]
+    pub fn transform_dist(&self, d: f64) -> f64 {
+        d * (self.paffine.xx.powi(2) + self.paffine.yy.powi(2)).sqrt() / f64::consts::SQRT_2
+    }
 }
 
 impl FilterInput {
@@ -631,181 +649,10 @@ impl From<cairo::Rectangle> for IRect {
     }
 }
 
-#[no_mangle]
-pub unsafe extern "C" fn rsvg_filter_context_get_paffine(
-    ctx: *const RsvgFilterContext,
-) -> cairo::Matrix {
-    assert!(!ctx.is_null());
-
-    (*ctx).paffine
-}
-
-#[no_mangle]
-pub unsafe extern "C" fn rsvg_filter_context_get_width(ctx: *const RsvgFilterContext) -> i32 {
-    assert!(!ctx.is_null());
-
-    (*ctx).source_surface.width()
-}
-
-#[no_mangle]
-pub unsafe extern "C" fn rsvg_filter_context_get_height(ctx: *const RsvgFilterContext) -> i32 {
-    assert!(!ctx.is_null());
-
-    (*ctx).source_surface.height()
-}
-
-#[no_mangle]
-pub unsafe extern "C" fn rsvg_filter_context_get_channelmap(
-    ctx: *const RsvgFilterContext,
-) -> *const i32 {
-    assert!(!ctx.is_null());
-
-    (*ctx).channelmap.as_ptr()
-}
-
-#[no_mangle]
-pub unsafe extern "C" fn rsvg_filter_context_get_source_surface(
-    ctx: *mut RsvgFilterContext,
-) -> *mut cairo_surface_t {
-    assert!(!ctx.is_null());
-
-    (*ctx).source_surface.to_glib_none().0
-}
-
-#[no_mangle]
-pub unsafe extern "C" fn rsvg_filter_store_output(
-    name: *mut GString,
-    result: RsvgFilterPrimitiveOutput,
-    ctx: *mut RsvgFilterContext,
-) {
-    assert!(!name.is_null());
-    assert!(!result.surface.is_null());
-    assert!(!ctx.is_null());
-
-    let name = from_glib_none((*name).str);
-
-    let surface: cairo::Surface = from_glib_full(result.surface);
-    assert_eq!(surface.get_type(), cairo::SurfaceType::Image);
-    let surface = cairo::ImageSurface::from(surface).unwrap();
-    let surface = SharedImageSurface::new(surface).unwrap();
-
-    let result = FilterResult {
-        name: Some(name),
-        output: FilterOutput {
-            surface,
-            bounds: result.bounds,
-        },
-    };
-
-    (*ctx).store_result(result);
-}
-
-#[no_mangle]
-pub unsafe extern "C" fn rsvg_filter_primitive_get_bounds(
-    primitive: *const RsvgFilterPrimitive,
-    ctx: *const RsvgFilterContext,
-    raw_draw_ctx: *mut RsvgDrawingCtx,
-) -> IRect {
-    assert!(!ctx.is_null());
-    assert!(!raw_draw_ctx.is_null());
-
-    let ctx = &*ctx;
-    let draw_ctx = &mut *(raw_draw_ctx as *mut DrawingCtx);
-
-    let mut x = None;
-    let mut y = None;
-    let mut width = None;
-    let mut height = None;
-
-    if !primitive.is_null() {
-        if (*primitive).x_specified != 0 {
-            x = Some((*primitive).x)
-        };
-
-        if (*primitive).y_specified != 0 {
-            y = Some((*primitive).y)
-        };
-
-        if (*primitive).width_specified != 0 {
-            width = Some((*primitive).width)
-        };
-
-        if (*primitive).height_specified != 0 {
-            height = Some((*primitive).height)
-        };
-    }
-
-    // Doesn't take referenced nodes into account, which is wrong.
-    BoundsBuilder::new(ctx, x, y, width, height).into_irect(draw_ctx)
-}
-
-#[no_mangle]
-pub unsafe extern "C" fn rsvg_filter_get_result(
-    name: *const GString,
-    ctx: *const RsvgFilterContext,
-    raw_draw_ctx: *const RsvgDrawingCtx,
-) -> RsvgFilterPrimitiveOutput {
-    assert!(!name.is_null());
-    assert!(!ctx.is_null());
-    assert!(!raw_draw_ctx.is_null());
-
-    let draw_ctx = &mut *(raw_draw_ctx as *mut DrawingCtx);
-
-    let name: String = from_glib_none((*name).str);
-    let input = match &name[..] {
-        "" | "none" => None,
-        "SourceGraphic" => Some(Input::SourceGraphic),
-        "SourceAlpha" => Some(Input::SourceAlpha),
-        "BackgroundImage" => Some(Input::BackgroundImage),
-        "BackgroundAlpha" => Some(Input::BackgroundAlpha),
-        "FillPaint" => Some(Input::FillPaint),
-        "StrokePaint" => Some(Input::StrokePaint),
-        _ => Some(Input::FilterOutput(name)),
-    };
-
-    let ctx = &*ctx;
-
-    match ctx.get_input(draw_ctx, input.as_ref()) {
-        None => RsvgFilterPrimitiveOutput {
-            surface: ptr::null_mut(),
-            bounds: IRect {
-                x0: 0,
-                x1: 0,
-                y0: 0,
-                y1: 0,
-            },
-        },
-        Some(input) => {
-            // HACK because to_glib_full() is unimplemented!() on ImageSurface.
-            let ptr = input.surface().to_glib_none().0;
-
-            let rv = RsvgFilterPrimitiveOutput {
-                surface: ptr,
-                bounds: match input {
-                    FilterInput::StandardInput(_) => ctx.effects_region().rect.unwrap().into(),
-                    FilterInput::PrimitiveOutput(FilterOutput { bounds, .. }) => bounds,
-                },
-            };
-
-            mem::forget(input);
-
-            rv
-        }
-    }
-}
-
-#[no_mangle]
-pub unsafe extern "C" fn rsvg_filter_get_in(
-    name: *const GString,
-    ctx: *const RsvgFilterContext,
-    raw_draw_ctx: *mut RsvgDrawingCtx,
-) -> *mut cairo_surface_t {
-    rsvg_filter_get_result(name, ctx, raw_draw_ctx).surface
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
+    use surface_utils::iterators::Pixels;
 
     #[test]
     fn test_extract_alpha() {
@@ -839,7 +686,7 @@ mod tests {
         }
 
         let surface = SharedImageSurface::new(surface).unwrap();
-        let alpha = SharedImageSurface::new(extract_alpha(&surface, BOUNDS).unwrap()).unwrap();
+        let alpha = SharedImageSurface::new(surface.extract_alpha(BOUNDS).unwrap()).unwrap();
 
         for (x, y, p, pa) in
             Pixels::new(&surface, FULL_BOUNDS).map(|(x, y, p)| (x, y, p, alpha.get_pixel(x, y)))
