@@ -12,7 +12,7 @@ use aspect_ratio::AspectRatio;
 use attributes::Attribute;
 use drawing_ctx::DrawingCtx;
 use handle::RsvgHandle;
-use node::{CascadedValues, NodeResult, NodeTrait, RsvgCNodeImpl, RsvgNode};
+use node::{CascadedValues, NodeResult, NodeTrait, RsvgNode};
 use parsers::parse;
 use property_bag::PropertyBag;
 use surface_utils::shared_surface::SharedImageSurface;
@@ -64,7 +64,7 @@ impl Image {
             cairo::Format::ARgb32,
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
-        ).map_err(FilterError::OutputSurfaceCreation)?;
+        ).map_err(FilterError::IntermediateSurfaceCreation)?;
 
         draw_ctx.get_cairo_context().set_matrix(ctx.paffine());
 
@@ -87,7 +87,7 @@ impl Image {
             cairo::Format::ARgb32,
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
-        ).map_err(FilterError::OutputSurfaceCreation)?;
+        ).map_err(FilterError::IntermediateSurfaceCreation)?;
 
         let cr = cairo::Context::new(&output_surface);
         cr.rectangle(
@@ -142,7 +142,7 @@ impl Image {
             cairo::Format::ARgb32,
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
-        ).map_err(FilterError::OutputSurfaceCreation)?;
+        ).map_err(FilterError::IntermediateSurfaceCreation)?;
 
         // TODO: this goes through a f64->i32->f64 conversion.
         let render_bounds = bounds_builder.into_irect_without_clipping(draw_ctx);
@@ -213,11 +213,6 @@ impl NodeTrait for Image {
 
         Ok(())
     }
-
-    #[inline]
-    fn get_c_impl(&self) -> *const RsvgCNodeImpl {
-        self.base.get_c_impl()
-    }
 }
 
 impl Filter for Image {
@@ -244,14 +239,15 @@ impl Filter for Image {
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: SharedImageSurface::new(output_surface).unwrap(),
+                surface: SharedImageSurface::new(output_surface)
+                    .map_err(FilterError::BadIntermediateSurfaceStatus)?,
                 bounds,
             },
         })
     }
 
     #[inline]
-    fn is_affected_by_color_interpolation_filters() -> bool {
+    fn is_affected_by_color_interpolation_filters(&self) -> bool {
         false
     }
 }
