@@ -5,13 +5,13 @@ use cairo::{self, ImageSurface};
 use attributes::Attribute;
 use drawing_ctx::DrawingCtx;
 use handle::RsvgHandle;
-use node::{NodeResult, NodeTrait, NodeType, RsvgCNodeImpl, RsvgNode};
+use node::{NodeResult, NodeTrait, NodeType, RsvgNode};
 use property_bag::PropertyBag;
 use surface_utils::shared_surface::SharedImageSurface;
 
 use super::context::{FilterContext, FilterOutput, FilterResult, IRect};
 use super::input::Input;
-use super::{make_result, Filter, FilterError, Primitive};
+use super::{Filter, FilterError, Primitive};
 
 /// The `feMerge` filter primitive.
 pub struct Merge {
@@ -53,11 +53,6 @@ impl NodeTrait for Merge {
     ) -> NodeResult {
         self.base.set_atts(node, handle, pbag)
     }
-
-    #[inline]
-    fn get_c_impl(&self) -> *const RsvgCNodeImpl {
-        self.base.get_c_impl()
-    }
 }
 
 impl NodeTrait for MergeNode {
@@ -89,13 +84,13 @@ impl MergeNode {
         bounds: IRect,
         output_surface: Option<ImageSurface>,
     ) -> Result<ImageSurface, FilterError> {
-        let input = make_result(ctx.get_input(draw_ctx, self.in_.borrow().as_ref()))?;
+        let input = ctx.get_input(draw_ctx, self.in_.borrow().as_ref())?;
 
         if output_surface.is_none() {
             return input
                 .surface()
                 .copy_surface(bounds)
-                .map_err(FilterError::OutputSurfaceCreation);
+                .map_err(FilterError::IntermediateSurfaceCreation);
         }
         let output_surface = output_surface.unwrap();
 
@@ -129,9 +124,9 @@ impl Filter for Merge {
             .children()
             .filter(|c| c.get_type() == NodeType::FilterPrimitiveMergeNode)
         {
-            bounds = bounds.add_input(&child.with_impl(|c: &MergeNode| {
-                make_result(ctx.get_input(draw_ctx, c.in_.borrow().as_ref()))
-            })?);
+            bounds = bounds.add_input(
+                &child.with_impl(|c: &MergeNode| ctx.get_input(draw_ctx, c.in_.borrow().as_ref()))?,
+            );
         }
         let bounds = bounds.into_irect(draw_ctx);
 
@@ -152,20 +147,21 @@ impl Filter for Merge {
                 cairo::Format::ARgb32,
                 ctx.source_graphic().width(),
                 ctx.source_graphic().height(),
-            ).map_err(FilterError::OutputSurfaceCreation)?,
+            ).map_err(FilterError::IntermediateSurfaceCreation)?,
         };
 
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: SharedImageSurface::new(output_surface).unwrap(),
+                surface: SharedImageSurface::new(output_surface)
+                    .map_err(FilterError::BadIntermediateSurfaceStatus)?,
                 bounds,
             },
         })
     }
 
     #[inline]
-    fn is_affected_by_color_interpolation_filters() -> bool {
+    fn is_affected_by_color_interpolation_filters(&self) -> bool {
         true
     }
 }
