@@ -1,19 +1,19 @@
 use std::cell::Cell;
 
-use cairo::{self, ImageSurface};
+use cairo::{self, ImageSurface, MatrixTrait};
 
 use attributes::Attribute;
 use drawing_ctx::DrawingCtx;
 use error::NodeError;
 use handle::RsvgHandle;
-use node::{NodeResult, NodeTrait, RsvgCNodeImpl, RsvgNode};
+use node::{NodeResult, NodeTrait, RsvgNode};
 use parsers;
 use property_bag::PropertyBag;
 use surface_utils::shared_surface::SharedImageSurface;
 use util::clamp;
 
 use super::context::{FilterContext, FilterOutput, FilterResult, IRect};
-use super::{make_result, Filter, FilterError, PrimitiveWithInput};
+use super::{Filter, FilterError, PrimitiveWithInput};
 
 /// The `feOffset` filter primitive.
 pub struct Offset {
@@ -57,11 +57,6 @@ impl NodeTrait for Offset {
 
         Ok(())
     }
-
-    #[inline]
-    fn get_c_impl(&self) -> *const RsvgCNodeImpl {
-        self.base.get_c_impl()
-    }
 }
 
 impl Filter for Offset {
@@ -71,7 +66,7 @@ impl Filter for Offset {
         ctx: &FilterContext,
         draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterResult, FilterError> {
-        let input = make_result(self.base.get_input(ctx, draw_ctx))?;
+        let input = self.base.get_input(ctx, draw_ctx)?;
         let bounds = self
             .base
             .get_bounds(ctx)
@@ -80,9 +75,7 @@ impl Filter for Offset {
 
         let dx = self.dx.get();
         let dy = self.dy.get();
-        let paffine = ctx.paffine();
-        let ox = paffine.xx * dx + paffine.xy * dy;
-        let oy = paffine.yx * dx + paffine.yy * dy;
+        let (ox, oy) = ctx.paffine().transform_distance(dx, dy);
 
         // output_bounds contains all pixels within bounds,
         // for which (x - ox) and (y - oy) also lie within bounds.
@@ -97,7 +90,7 @@ impl Filter for Offset {
             cairo::Format::ARgb32,
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
-        ).map_err(FilterError::OutputSurfaceCreation)?;
+        ).map_err(FilterError::IntermediateSurfaceCreation)?;
 
         {
             let cr = cairo::Context::new(&output_surface);
@@ -116,14 +109,15 @@ impl Filter for Offset {
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: SharedImageSurface::new(output_surface).unwrap(),
+                surface: SharedImageSurface::new(output_surface)
+                    .map_err(FilterError::BadIntermediateSurfaceStatus)?,
                 bounds,
             },
         })
     }
 
     #[inline]
-    fn is_affected_by_color_interpolation_filters() -> bool {
+    fn is_affected_by_color_interpolation_filters(&self) -> bool {
         false
     }
 }
