@@ -1,12 +1,12 @@
 use cairo::{self, MatrixTrait};
-use cssparser;
+use cssparser::{self, CowRcStr, Parser, Token};
 
 use std::cell::RefCell;
 
 use attributes::Attribute;
 use bbox::*;
 use coord_units::CoordUnits;
-use drawing_ctx::{self, AcquiredNode, RsvgDrawingCtx};
+use drawing_ctx::{AcquiredNode, DrawingCtx};
 use error::*;
 use handle::RsvgHandle;
 use length::*;
@@ -39,15 +39,24 @@ impl Parse for SpreadMethod {
     type Data = ();
     type Err = AttributeError;
 
-    fn parse(s: &str, _: ()) -> Result<SpreadMethod, AttributeError> {
-        match s {
-            "pad" => Ok(SpreadMethod::Pad),
-            "reflect" => Ok(SpreadMethod::Reflect),
-            "repeat" => Ok(SpreadMethod::Repeat),
-            _ => Err(AttributeError::Parse(ParseError::new(
-                "expected 'pad' | 'reflect' | 'repeat'",
-            ))),
-        }
+    fn parse(parser: &mut Parser, _: ()) -> Result<SpreadMethod, AttributeError> {
+        let loc = parser.current_source_location();
+
+        parser
+            .expect_ident()
+            .and_then(|cow| match cow.as_ref() {
+                "pad" => Ok(SpreadMethod::Pad),
+                "reflect" => Ok(SpreadMethod::Reflect),
+                "repeat" => Ok(SpreadMethod::Repeat),
+                _ => Err(
+                    loc.new_basic_unexpected_token_error(Token::Ident(CowRcStr::from(
+                        cow.as_ref().to_string(),
+                    ))),
+                ),
+            })
+            .map_err(|_| {
+                AttributeError::Parse(ParseError::new("expected 'pad' | 'reflect' | 'repeat'"))
+            })
     }
 }
 
@@ -240,10 +249,10 @@ impl GradientVariant {
         // https://www.w3.org/TR/SVG/pservers.html#LinearGradients
 
         GradientVariant::Linear {
-            x1: Some(RsvgLength::parse("0%", LengthDir::Horizontal).unwrap()),
-            y1: Some(RsvgLength::parse("0%", LengthDir::Vertical).unwrap()),
-            x2: Some(RsvgLength::parse("100%", LengthDir::Horizontal).unwrap()),
-            y2: Some(RsvgLength::parse("0%", LengthDir::Vertical).unwrap()),
+            x1: Some(RsvgLength::parse_str("0%", LengthDir::Horizontal).unwrap()),
+            y1: Some(RsvgLength::parse_str("0%", LengthDir::Vertical).unwrap()),
+            x2: Some(RsvgLength::parse_str("100%", LengthDir::Horizontal).unwrap()),
+            y2: Some(RsvgLength::parse_str("0%", LengthDir::Vertical).unwrap()),
         }
     }
 
@@ -251,9 +260,9 @@ impl GradientVariant {
         // https://www.w3.org/TR/SVG/pservers.html#RadialGradients
 
         GradientVariant::Radial {
-            cx: Some(RsvgLength::parse("50%", LengthDir::Horizontal).unwrap()),
-            cy: Some(RsvgLength::parse("50%", LengthDir::Vertical).unwrap()),
-            r: Some(RsvgLength::parse("50%", LengthDir::Both).unwrap()),
+            cx: Some(RsvgLength::parse_str("50%", LengthDir::Horizontal).unwrap()),
+            cy: Some(RsvgLength::parse_str("50%", LengthDir::Vertical).unwrap()),
+            r: Some(RsvgLength::parse_str("50%", LengthDir::Both).unwrap()),
 
             fx: None,
             fy: None,
@@ -397,8 +406,8 @@ impl Gradient {
     }
 }
 
-fn acquire_gradient(draw_ctx: *mut RsvgDrawingCtx, name: &str) -> Option<AcquiredNode> {
-    drawing_ctx::get_acquired_node(draw_ctx, name).and_then(|acquired| {
+fn acquire_gradient<'a>(draw_ctx: &'a mut DrawingCtx, name: &str) -> Option<AcquiredNode> {
+    draw_ctx.get_acquired_node(name).and_then(|acquired| {
         // FIXME: replace with .filter() once Option.filter() becomes stable
         let node = acquired.get();
         if node.get_type() == NodeType::LinearGradient
@@ -411,7 +420,7 @@ fn acquire_gradient(draw_ctx: *mut RsvgDrawingCtx, name: &str) -> Option<Acquire
     })
 }
 
-fn resolve_gradient(gradient: &Gradient, draw_ctx: *mut RsvgDrawingCtx) -> Gradient {
+fn resolve_gradient(gradient: &Gradient, draw_ctx: &mut DrawingCtx) -> Gradient {
     let mut result = gradient.clone();
 
     while !result.is_resolved() {
@@ -440,12 +449,12 @@ fn resolve_gradient(gradient: &Gradient, draw_ctx: *mut RsvgDrawingCtx) -> Gradi
 
 fn set_common_on_pattern<P: cairo::Pattern + cairo::Gradient>(
     gradient: &Gradient,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     pattern: &mut P,
     bbox: &BoundingBox,
     opacity: &UnitInterval,
 ) {
-    let cr = drawing_ctx::get_cairo_context(draw_ctx);
+    let cr = draw_ctx.get_cairo_context();
 
     let mut affine = gradient.common.affine.unwrap();
 
@@ -478,7 +487,7 @@ fn set_common_on_pattern<P: cairo::Pattern + cairo::Gradient>(
 fn set_linear_gradient_on_pattern(
     gradient: &Gradient,
     values: &ComputedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     bbox: &BoundingBox,
     opacity: &UnitInterval,
 ) -> bool {
@@ -486,7 +495,7 @@ fn set_linear_gradient_on_pattern(
         let units = gradient.common.units.unwrap();
 
         if units == GradientUnits(CoordUnits::ObjectBoundingBox) {
-            drawing_ctx::push_view_box(draw_ctx, 1.0, 1.0);
+            draw_ctx.push_view_box(1.0, 1.0);
         }
 
         let mut pattern = cairo::LinearGradient::new(
@@ -497,7 +506,7 @@ fn set_linear_gradient_on_pattern(
         );
 
         if units == GradientUnits(CoordUnits::ObjectBoundingBox) {
-            drawing_ctx::pop_view_box(draw_ctx);
+            draw_ctx.pop_view_box();
         }
 
         set_common_on_pattern(gradient, draw_ctx, &mut pattern, bbox, opacity);
@@ -558,7 +567,7 @@ fn fix_focus_point(mut fx: f64, mut fy: f64, cx: f64, cy: f64, radius: f64) -> (
 fn set_radial_gradient_on_pattern(
     gradient: &Gradient,
     values: &ComputedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     bbox: &BoundingBox,
     opacity: &UnitInterval,
 ) -> bool {
@@ -566,7 +575,7 @@ fn set_radial_gradient_on_pattern(
         let units = gradient.common.units.unwrap();
 
         if units == GradientUnits(CoordUnits::ObjectBoundingBox) {
-            drawing_ctx::push_view_box(draw_ctx, 1.0, 1.0);
+            draw_ctx.push_view_box(1.0, 1.0);
         }
 
         let n_cx = cx.as_ref().unwrap().normalize(values, draw_ctx);
@@ -580,7 +589,7 @@ fn set_radial_gradient_on_pattern(
         let mut pattern = cairo::RadialGradient::new(new_fx, new_fy, 0.0, n_cx, n_cy, n_r);
 
         if units == GradientUnits(CoordUnits::ObjectBoundingBox) {
-            drawing_ctx::pop_view_box(draw_ctx);
+            draw_ctx.pop_view_box();
         }
 
         set_common_on_pattern(gradient, draw_ctx, &mut pattern, bbox, opacity);
@@ -594,7 +603,7 @@ fn set_radial_gradient_on_pattern(
 fn set_pattern_on_draw_context(
     gradient: &Gradient,
     values: &ComputedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     opacity: &UnitInterval,
     bbox: &BoundingBox,
 ) -> bool {
@@ -660,31 +669,31 @@ impl NodeTrait for NodeGradient {
             match attr {
                 // Attributes common to linear and radial gradients
                 Attribute::GradientUnits => {
-                    g.common.units = Some(parse("gradientUnits", value, (), None)?)
+                    g.common.units = Some(parse("gradientUnits", value, ())?)
                 }
 
                 Attribute::GradientTransform => {
-                    g.common.affine = Some(parse("gradientTransform", value, (), None)?)
+                    g.common.affine = Some(parse("gradientTransform", value, ())?)
                 }
 
                 Attribute::SpreadMethod => {
-                    g.common.spread = Some(parse("spreadMethod", value, (), None)?)
+                    g.common.spread = Some(parse("spreadMethod", value, ())?)
                 }
 
                 Attribute::XlinkHref => g.common.fallback = Some(value.to_owned()),
 
                 // Attributes specific to each gradient type.  The defaults mandated by the spec
                 // are in GradientVariant::resolve_from_defaults()
-                Attribute::X1 => x1 = Some(parse("x1", value, LengthDir::Horizontal, None)?),
-                Attribute::Y1 => y1 = Some(parse("y1", value, LengthDir::Vertical, None)?),
-                Attribute::X2 => x2 = Some(parse("x2", value, LengthDir::Horizontal, None)?),
-                Attribute::Y2 => y2 = Some(parse("y2", value, LengthDir::Vertical, None)?),
+                Attribute::X1 => x1 = Some(parse("x1", value, LengthDir::Horizontal)?),
+                Attribute::Y1 => y1 = Some(parse("y1", value, LengthDir::Vertical)?),
+                Attribute::X2 => x2 = Some(parse("x2", value, LengthDir::Horizontal)?),
+                Attribute::Y2 => y2 = Some(parse("y2", value, LengthDir::Vertical)?),
 
-                Attribute::Cx => cx = Some(parse("cx", value, LengthDir::Horizontal, None)?),
-                Attribute::Cy => cy = Some(parse("cy", value, LengthDir::Vertical, None)?),
-                Attribute::R => r = Some(parse("r", value, LengthDir::Both, None)?),
-                Attribute::Fx => fx = Some(parse("fx", value, LengthDir::Horizontal, None)?),
-                Attribute::Fy => fy = Some(parse("fy", value, LengthDir::Vertical, None)?),
+                Attribute::Cx => cx = Some(parse("cx", value, LengthDir::Horizontal)?),
+                Attribute::Cy => cy = Some(parse("cy", value, LengthDir::Vertical)?),
+                Attribute::R => r = Some(parse("r", value, LengthDir::Both)?),
+                Attribute::Fx => fx = Some(parse("fx", value, LengthDir::Horizontal)?),
+                Attribute::Fy => fy = Some(parse("fy", value, LengthDir::Vertical)?),
 
                 _ => (),
             }
@@ -709,7 +718,7 @@ impl NodeTrait for NodeGradient {
 fn resolve_fallbacks_and_set_pattern(
     gradient: &Gradient,
     values: &ComputedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     opacity: &UnitInterval,
     bbox: &BoundingBox,
 ) -> bool {
@@ -725,7 +734,7 @@ fn resolve_fallbacks_and_set_pattern(
 
 pub fn gradient_resolve_fallbacks_and_set_pattern(
     node: &RsvgNode,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     opacity: &UnitInterval,
     bbox: &BoundingBox,
 ) -> bool {
@@ -753,13 +762,16 @@ mod tests {
 
     #[test]
     fn parses_spread_method() {
-        assert_eq!(SpreadMethod::parse("pad", ()), Ok(SpreadMethod::Pad));
+        assert_eq!(SpreadMethod::parse_str("pad", ()), Ok(SpreadMethod::Pad));
         assert_eq!(
-            SpreadMethod::parse("reflect", ()),
+            SpreadMethod::parse_str("reflect", ()),
             Ok(SpreadMethod::Reflect)
         );
-        assert_eq!(SpreadMethod::parse("repeat", ()), Ok(SpreadMethod::Repeat));
-        assert!(SpreadMethod::parse("foobar", ()).is_err());
+        assert_eq!(
+            SpreadMethod::parse_str("repeat", ()),
+            Ok(SpreadMethod::Repeat)
+        );
+        assert!(SpreadMethod::parse_str("foobar", ()).is_err());
     }
 
     #[test]
