@@ -10,6 +10,7 @@ extern crate glib;
 extern crate glib_sys;
 extern crate itertools;
 extern crate libc;
+extern crate owning_ref;
 extern crate pango;
 extern crate pango_cairo_sys;
 extern crate pango_sys;
@@ -22,9 +23,10 @@ extern crate lazy_static;
 #[macro_use]
 extern crate downcast_rs;
 
-pub use attributes::rsvg_attribute_from_name;
+#[macro_use]
+extern crate rulinalg;
 
-pub use cnode::{rsvg_rust_cnode_get_impl, rsvg_rust_cnode_new};
+pub use attributes::rsvg_attribute_from_name;
 
 pub use color::{rsvg_css_parse_color, ColorKind, ColorSpec};
 
@@ -36,18 +38,6 @@ pub use drawing_ctx::{
     rsvg_drawing_ctx_new,
 };
 
-pub use filters::context::{
-    rsvg_filter_context_get_channelmap,
-    rsvg_filter_context_get_height,
-    rsvg_filter_context_get_paffine,
-    rsvg_filter_context_get_source_surface,
-    rsvg_filter_context_get_width,
-    rsvg_filter_get_in,
-    rsvg_filter_get_result,
-    rsvg_filter_primitive_get_bounds,
-    rsvg_filter_store_output,
-};
-
 pub use length::{
     rsvg_length_hand_normalize,
     rsvg_length_normalize,
@@ -57,7 +47,7 @@ pub use length::{
     RsvgLength,
 };
 
-pub use load::rsvg_load_new_node;
+pub use load::{rsvg_load_new_node, rsvg_load_set_node_atts};
 
 pub use node::{
     rsvg_node_add_child,
@@ -71,13 +61,11 @@ pub use node::{
     rsvg_node_ref,
     rsvg_node_register_in_defs,
     rsvg_node_set_attribute_parse_error,
-    rsvg_node_set_atts,
-    rsvg_node_set_overridden_properties,
     rsvg_node_unref,
     rsvg_root_node_cascade,
 };
 
-pub use parsers::{rsvg_css_parse_number_list, rsvg_css_parse_number_optional_number};
+pub use parsers::rsvg_css_parse_number_optional_number;
 
 pub use property_bag::{
     rsvg_property_bag_free,
@@ -89,7 +77,6 @@ pub use property_bag::{
 
 pub use state::{
     rsvg_computed_values_get_lighting_color_argb,
-    rsvg_parse_style_attrs,
     rsvg_state_free,
     rsvg_state_new,
     rsvg_state_parse_style_pair,
@@ -99,8 +86,6 @@ pub use structure::{rsvg_node_svg_apply_atts, rsvg_node_svg_get_size, rsvg_node_
 
 pub use text::{rsvg_node_chars_append, rsvg_node_chars_new};
 
-pub use viewbox::RsvgViewBox;
-
 #[macro_use]
 mod coord_units;
 
@@ -114,11 +99,9 @@ mod aspect_ratio;
 mod attributes;
 mod bbox;
 mod clip_path;
-mod cnode;
 mod color;
 mod cond;
 mod defs;
-mod draw;
 mod drawing_ctx;
 mod error;
 pub mod filters;
