@@ -35,10 +35,6 @@ use util::utf8_cstr;
 // of nodes.
 pub type RsvgNode = Rc<Node>;
 
-// A *const RsvgCNodeImpl is just an opaque pointer to the C code's
-// struct for a particular node type.
-pub enum RsvgCNodeImpl {}
-
 /// Can obtain computed values from a node
 ///
 /// In our tree of SVG elements (Node in our parlance), each node stores a `ComputedValues` that
@@ -136,10 +132,6 @@ pub trait NodeTrait: Downcast {
     ) {
         // by default nodes don't draw themselves
     }
-
-    fn get_c_impl(&self) -> *const RsvgCNodeImpl {
-        unreachable!(); // no Rust nodes should have this method called for them
-    }
 }
 
 impl_downcast!(NodeTrait);
@@ -235,12 +227,12 @@ pub enum NodeType {
     FilterPrimitiveConvolveMatrix,
     FilterPrimitiveDiffuseLighting,
     FilterPrimitiveDisplacementMap,
-    FilterPrimitiveErode,
     FilterPrimitiveFlood,
     FilterPrimitiveGaussianBlur,
     FilterPrimitiveImage,
     FilterPrimitiveMerge,
     FilterPrimitiveMergeNode,
+    FilterPrimitiveMorphology,
     FilterPrimitiveOffset,
     FilterPrimitiveSpecularLighting,
     FilterPrimitiveTile,
@@ -334,7 +326,7 @@ impl Node {
         let mut desc = Some(descendant.clone());
 
         while let Some(ref d) = desc.clone() {
-            if rc_node_ptr_eq(&ancestor, d) {
+            if Rc::ptr_eq(&ancestor, d) {
                 return true;
             }
 
@@ -456,10 +448,6 @@ impl Node {
         self.result.borrow().clone()
     }
 
-    pub fn get_c_impl(&self) -> *const RsvgCNodeImpl {
-        self.node_impl.get_c_impl()
-    }
-
     pub fn with_impl<T, F, U>(&self, f: F) -> U
     where
         T: NodeTrait,
@@ -641,15 +629,6 @@ pub extern "C" fn rsvg_node_unref(raw_node: *mut RsvgNode) -> *mut RsvgNode {
     ptr::null_mut()
 }
 
-// See https://github.com/rust-lang/rust/issues/36497 - this is what
-// added Rc::ptr_eq(), but we don't want to depend on unstable Rust
-// just yet.
-pub fn rc_node_ptr_eq<T: ?Sized>(this: &Rc<T>, other: &Rc<T>) -> bool {
-    let this_ptr: *const T = &**this;
-    let other_ptr: *const T = &**other;
-    this_ptr == other_ptr
-}
-
 #[no_mangle]
 pub extern "C" fn rsvg_node_is_same(
     raw_node1: *const RsvgNode,
@@ -661,7 +640,7 @@ pub extern "C" fn rsvg_node_is_same(
         let node1: &RsvgNode = unsafe { &*raw_node1 };
         let node2: &RsvgNode = unsafe { &*raw_node2 };
 
-        rc_node_ptr_eq(node1, node2)
+        Rc::ptr_eq(node1, node2)
     } else {
         false
     };
@@ -689,30 +668,6 @@ pub extern "C" fn rsvg_node_register_in_defs(raw_node: *const RsvgNode, defs: *m
     node.register(node, defs);
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_node_set_atts(
-    raw_node: *mut RsvgNode,
-    handle: *const RsvgHandle,
-    pbag: *const PropertyBag,
-) {
-    assert!(!raw_node.is_null());
-    assert!(!pbag.is_null());
-
-    let node: &RsvgNode = unsafe { &*raw_node };
-    let pbag = unsafe { &*pbag };
-
-    node.set_atts(node, handle, pbag);
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_node_set_overridden_properties(raw_node: *mut RsvgNode) {
-    assert!(!raw_node.is_null());
-
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    node.set_overridden_properties();
-}
-
 #[no_mangle]
 pub extern "C" fn rsvg_node_set_attribute_parse_error(
     raw_node: *const RsvgNode,
@@ -972,12 +927,12 @@ mod tests {
         let c = children.next();
         assert!(c.is_some());
         let c = c.unwrap();
-        assert!(rc_node_ptr_eq(&c, &child));
+        assert!(Rc::ptr_eq(&c, &child));
 
         let c = children.next_back();
         assert!(c.is_some());
         let c = c.unwrap();
-        assert!(rc_node_ptr_eq(&c, &second_child));
+        assert!(Rc::ptr_eq(&c, &second_child));
 
         assert!(children.next().is_none());
         assert!(children.next_back().is_none());
@@ -1020,12 +975,12 @@ mod tests {
 
         let result: bool = from_glib(rsvg_node_children_iter_next(iter, &mut c));
         assert_eq!(result, true);
-        assert!(rc_node_ptr_eq(unsafe { &*c }, &child));
+        assert!(Rc::ptr_eq(unsafe { &*c }, &child));
         rsvg_node_unref(c);
 
         let result: bool = from_glib(rsvg_node_children_iter_next_back(iter, &mut c));
         assert_eq!(result, true);
-        assert!(rc_node_ptr_eq(unsafe { &*c }, &second_child));
+        assert!(Rc::ptr_eq(unsafe { &*c }, &second_child));
         rsvg_node_unref(c);
 
         let result: bool = from_glib(rsvg_node_children_iter_next(iter, &mut c));
