@@ -1,11 +1,10 @@
 use cairo;
-use cairo_sys;
-use glib::translate::*;
 
 use std::f64;
 use std::f64::consts::*;
 
 use float_eq_cairo::ApproxEqCairo;
+use util::clamp;
 
 #[derive(Debug, Copy, Clone, PartialEq)]
 pub struct LargeArc(pub bool);
@@ -25,14 +24,13 @@ pub enum PathCommand {
     ClosePath,
 }
 
-#[repr(C)]
-pub struct RsvgPathBuilder {
+pub struct PathBuilder {
     path_commands: Vec<PathCommand>,
 }
 
-impl Default for RsvgPathBuilder {
-    fn default() -> RsvgPathBuilder {
-        RsvgPathBuilder {
+impl Default for PathBuilder {
+    fn default() -> PathBuilder {
+        PathBuilder {
             path_commands: Vec::new(),
         }
     }
@@ -60,9 +58,9 @@ impl PathCommand {
     }
 }
 
-impl RsvgPathBuilder {
-    pub fn new() -> RsvgPathBuilder {
-        RsvgPathBuilder::default()
+impl PathBuilder {
+    pub fn new() -> PathBuilder {
+        PathBuilder::default()
     }
 
     pub fn move_to(&mut self, x: f64, y: f64) {
@@ -285,36 +283,13 @@ impl RsvgPathBuilder {
     }
 }
 
-fn clamp(val: f64, low: f64, high: f64) -> f64 {
-    if val < low {
-        low
-    } else if val > high {
-        high
-    } else {
-        val
-    }
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_path_builder_add_to_cairo_context(
-    raw_builder: *mut RsvgPathBuilder,
-    raw_cr: *mut cairo_sys::cairo_t,
-) {
-    assert!(!raw_builder.is_null());
-    assert!(!raw_cr.is_null());
-
-    let builder: &mut RsvgPathBuilder = unsafe { &mut (*raw_builder) };
-    let cr = unsafe { cairo::Context::from_glib_none(raw_cr) };
-    builder.to_cairo(&cr);
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
 
     #[test]
     fn survives_degenerate_arcs() {
-        let mut builder = RsvgPathBuilder::new();
+        let mut builder = PathBuilder::new();
         builder.move_to(0.0, 0.0);
         builder.arc(
             0.0,
