@@ -9,12 +9,12 @@ use aspect_ratio::*;
 use attributes::Attribute;
 use bbox::*;
 use coord_units::CoordUnits;
-use drawing_ctx::{self, RsvgDrawingCtx};
+use drawing_ctx::DrawingCtx;
 use float_eq_cairo::ApproxEqCairo;
 use handle::RsvgHandle;
 use length::*;
 use node::*;
-use parsers::parse;
+use parsers::{parse, parse_and_validate};
 use property_bag::PropertyBag;
 use state::ComputedValues;
 use viewbox::*;
@@ -180,43 +180,43 @@ impl NodeTrait for NodePattern {
 
         for (_key, attr, value) in pbag.iter() {
             match attr {
-                Attribute::PatternUnits => p.units = Some(parse("patternUnits", value, (), None)?),
+                Attribute::PatternUnits => p.units = Some(parse("patternUnits", value, ())?),
 
                 Attribute::PatternContentUnits => {
-                    p.content_units = Some(parse("patternContentUnits", value, (), None)?)
+                    p.content_units = Some(parse("patternContentUnits", value, ())?)
                 }
 
-                Attribute::ViewBox => p.vbox = Some(Some(parse("viewBox", value, (), None)?)),
+                Attribute::ViewBox => p.vbox = Some(Some(parse("viewBox", value, ())?)),
 
                 Attribute::PreserveAspectRatio => {
-                    p.preserve_aspect_ratio = Some(parse("preserveAspectRatio", value, (), None)?)
+                    p.preserve_aspect_ratio = Some(parse("preserveAspectRatio", value, ())?)
                 }
 
                 Attribute::PatternTransform => {
-                    p.affine = Some(parse("patternTransform", value, (), None)?)
+                    p.affine = Some(parse("patternTransform", value, ())?)
                 }
 
                 Attribute::XlinkHref => p.fallback = Some(value.to_owned()),
 
-                Attribute::X => p.x = Some(parse("x", value, LengthDir::Horizontal, None)?),
+                Attribute::X => p.x = Some(parse("x", value, LengthDir::Horizontal)?),
 
-                Attribute::Y => p.y = Some(parse("y", value, LengthDir::Vertical, None)?),
+                Attribute::Y => p.y = Some(parse("y", value, LengthDir::Vertical)?),
 
                 Attribute::Width => {
-                    p.width = Some(parse(
+                    p.width = Some(parse_and_validate(
                         "width",
                         value,
                         LengthDir::Horizontal,
-                        Some(RsvgLength::check_nonnegative),
+                        RsvgLength::check_nonnegative,
                     )?)
                 }
 
                 Attribute::Height => {
-                    p.height = Some(parse(
+                    p.height = Some(parse_and_validate(
                         "height",
                         value,
                         LengthDir::Vertical,
-                        Some(RsvgLength::check_nonnegative),
+                        RsvgLength::check_nonnegative,
                     )?)
                 }
 
@@ -228,26 +228,20 @@ impl NodeTrait for NodePattern {
     }
 }
 
-fn resolve_pattern(pattern: &Pattern, draw_ctx: *mut RsvgDrawingCtx) -> Pattern {
+fn resolve_pattern(pattern: &Pattern, draw_ctx: &mut DrawingCtx) -> Pattern {
     let mut result = pattern.clone();
 
     while !result.is_resolved() {
-        result
-            .fallback
-            .as_ref()
-            .and_then(|fallback_name| {
-                drawing_ctx::get_acquired_node_of_type(draw_ctx, &fallback_name, NodeType::Pattern)
-            })
-            .and_then(|acquired| {
-                acquired.get().with_impl(|i: &NodePattern| {
-                    result.resolve_from_fallback(&*i.pattern.borrow())
-                });
-                Some(())
-            })
-            .or_else(|| {
-                result.resolve_from_defaults();
-                Some(())
-            });
+        if let Some(acquired) = draw_ctx.get_acquired_node_of_type(
+            result.fallback.as_ref().map(String::as_ref),
+            NodeType::Pattern,
+        ) {
+            acquired
+                .get()
+                .with_impl(|i: &NodePattern| result.resolve_from_fallback(&*i.pattern.borrow()));
+        } else {
+            result.resolve_from_defaults();
+        }
     }
 
     result
@@ -256,7 +250,7 @@ fn resolve_pattern(pattern: &Pattern, draw_ctx: *mut RsvgDrawingCtx) -> Pattern
 fn set_pattern_on_draw_context(
     pattern: &Pattern,
     values: &ComputedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     bbox: &BoundingBox,
 ) -> bool {
     assert!(pattern.is_resolved());
@@ -274,7 +268,7 @@ fn set_pattern_on_draw_context(
     let preserve_aspect_ratio = pattern.preserve_aspect_ratio.unwrap();
 
     if units == PatternUnits(CoordUnits::ObjectBoundingBox) {
-        drawing_ctx::push_view_box(draw_ctx, 1.0, 1.0);
+        draw_ctx.push_view_box(1.0, 1.0);
     }
 
     let pattern_x = pattern.x.unwrap().normalize(values, draw_ctx);
@@ -283,7 +277,7 @@ fn set_pattern_on_draw_context(
     let pattern_height = pattern.height.unwrap().normalize(values, draw_ctx);
 
     if units == PatternUnits(CoordUnits::ObjectBoundingBox) {
-        drawing_ctx::pop_view_box(draw_ctx);
+        draw_ctx.pop_view_box();
     }
 
     // Work out the size of the rectangle so it takes into account the object bounding box
@@ -304,7 +298,7 @@ fn set_pattern_on_draw_context(
         }
     }
 
-    let cr = drawing_ctx::get_cairo_context(draw_ctx);
+    let cr = draw_ctx.get_cairo_context();
     let affine = cr.get_matrix();
     let taffine = cairo::Matrix::multiply(&pattern_affine, &affine);
 
@@ -365,7 +359,7 @@ fn set_pattern_on_draw_context(
 
         caffine = cairo::Matrix::new(w / vbox.0.width, 0.0, 0.0, h / vbox.0.height, x, y);
 
-        drawing_ctx::push_view_box(draw_ctx, vbox.0.width, vbox.0.height);
+        draw_ctx.push_view_box(vbox.0.width, vbox.0.height);
         pushed_view_box = true;
     } else if content_units == PatternContentUnits(CoordUnits::ObjectBoundingBox) {
         // If coords are in terms of the bounding box, use them
@@ -374,7 +368,7 @@ fn set_pattern_on_draw_context(
         caffine = cairo::Matrix::identity();
         caffine.scale(bbrect.width, bbrect.height);
 
-        drawing_ctx::push_view_box(draw_ctx, 1.0, 1.0);
+        draw_ctx.push_view_box(1.0, 1.0);
         pushed_view_box = true;
     } else {
         caffine = cairo::Matrix::identity();
@@ -394,7 +388,7 @@ fn set_pattern_on_draw_context(
 
     // Draw to another surface
 
-    let cr_save = drawing_ctx::get_cairo_context(draw_ctx);
+    let cr_save = draw_ctx.get_cairo_context();
 
     let surface = cr_save
         .get_target()
@@ -402,7 +396,7 @@ fn set_pattern_on_draw_context(
 
     let cr_pattern = cairo::Context::new(&surface);
 
-    drawing_ctx::set_cairo_context(draw_ctx, &cr_pattern);
+    draw_ctx.set_cairo_context(&cr_pattern);
 
     // Set up transformations to be determined by the contents units
 
@@ -411,19 +405,18 @@ fn set_pattern_on_draw_context(
     let pattern_cascaded = pattern_node.get_cascaded_values();
     let pattern_values = pattern_cascaded.get();
 
-    drawing_ctx::push_discrete_layer(draw_ctx, pattern_values, false);
-
     cr_pattern.set_matrix(caffine);
-    pattern_node.draw_children(&pattern_node, &pattern_cascaded, draw_ctx, false, false);
 
-    drawing_ctx::pop_discrete_layer(draw_ctx, &pattern_node, pattern_values, false);
+    draw_ctx.with_discrete_layer(&pattern_node, pattern_values, false, &mut |dc| {
+        pattern_node.draw_children(&pattern_cascaded, dc, false);
+    });
 
     // Return to the original coordinate system and rendering context
 
-    drawing_ctx::set_cairo_context(draw_ctx, &cr_save);
+    draw_ctx.set_cairo_context(&cr_save);
 
     if pushed_view_box {
-        drawing_ctx::pop_view_box(draw_ctx);
+        draw_ctx.pop_view_box();
     }
 
     // Set the final surface as a Cairo pattern into the Cairo context
@@ -445,7 +438,7 @@ fn set_pattern_on_draw_context(
 fn resolve_fallbacks_and_set_pattern(
     pattern: &Pattern,
     values: &ComputedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     bbox: &BoundingBox,
 ) -> bool {
     let resolved = resolve_pattern(pattern, draw_ctx);
@@ -455,7 +448,7 @@ fn resolve_fallbacks_and_set_pattern(
 
 pub fn pattern_resolve_fallbacks_and_set_pattern(
     node: &RsvgNode,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     bbox: &BoundingBox,
 ) -> bool {
     assert!(node.get_type() == NodeType::Pattern);
