@@ -5,7 +5,6 @@ use std::cell::{Cell, RefCell};
 use std::str;
 
 use attributes::Attribute;
-use draw::draw_pango_layout;
 use drawing_ctx::DrawingCtx;
 use handle::RsvgHandle;
 use length::*;
@@ -94,10 +93,10 @@ impl NodeChars {
         let offset = baseline + values.baseline_shift.0.normalize(values, draw_ctx);
 
         if values.text_gravity_is_vertical() {
-            draw_pango_layout(draw_ctx, values, &layout, *x + offset, *y, clipping);
+            draw_ctx.draw_pango_layout(&layout, values, *x + offset, *y, clipping);
             *y += f64::from(width) / f64::from(pango::SCALE);
         } else {
-            draw_pango_layout(draw_ctx, values, &layout, *x, *y - offset, clipping);
+            draw_ctx.draw_pango_layout(&layout, values, *x, *y - offset, clipping);
             *x += f64::from(width) / f64::from(pango::SCALE);
         }
     }
@@ -506,9 +505,8 @@ fn create_pango_layout(
 
     font_desc.set_stretch(pango::Stretch::from(values.font_stretch));
 
-    let (_, dpi_y) = draw_ctx.get_dpi();
     font_desc.set_size(to_pango_units(
-        values.font_size.0.normalize(values, draw_ctx) / dpi_y * 72.0,
+        values.font_size.0.normalize(values, draw_ctx),
     ));
 
     let layout = pango::Layout::new(&pango_context);
