@@ -3,7 +3,7 @@ use cairo;
 use std::f64::consts::*;
 
 use cairo::MatrixTrait;
-use cssparser::{ParseError as CssParseError, Parser, ParserInput, Token};
+use cssparser::{ParseError as CssParseError, Parser, Token};
 
 use error::*;
 use parsers::{optional_comma, Parse, ParseError};
@@ -12,8 +12,8 @@ impl Parse for cairo::Matrix {
     type Data = ();
     type Err = AttributeError;
 
-    fn parse(s: &str, _: ()) -> Result<cairo::Matrix, AttributeError> {
-        let matrix = parse_transform_list(s)?;
+    fn parse(parser: &mut Parser, _: ()) -> Result<cairo::Matrix, AttributeError> {
+        let matrix = parse_transform_list(parser)?;
 
         matrix
             .try_invert()
@@ -26,10 +26,7 @@ impl Parse for cairo::Matrix {
 // Its operataion and grammar are described here:
 // https://www.w3.org/TR/SVG/coords.html#TransformAttribute
 
-fn parse_transform_list(s: &str) -> Result<cairo::Matrix, AttributeError> {
-    let mut input = ParserInput::new(s);
-    let mut parser = Parser::new(&mut input);
-
+fn parse_transform_list(parser: &mut Parser) -> Result<cairo::Matrix, AttributeError> {
     let mut matrix = cairo::Matrix::identity();
 
     loop {
@@ -37,10 +34,10 @@ fn parse_transform_list(s: &str) -> Result<cairo::Matrix, AttributeError> {
             break;
         }
 
-        let m = parse_transform_command(&mut parser)?;
+        let m = parse_transform_command(parser)?;
         matrix = cairo::Matrix::multiply(&m, &matrix);
 
-        optional_comma(&mut parser);
+        optional_comma(parser);
     }
 
     Ok(matrix)
@@ -208,7 +205,7 @@ mod tests {
     use super::*;
 
     fn parse_transform(s: &str) -> Result<cairo::Matrix, AttributeError> {
-        cairo::Matrix::parse(s, ())
+        cairo::Matrix::parse_str(s, ())
     }
 
     #[test]
